/* This is a script that lists the messages in the current
 * message sub-group, with the following features:
 * - Forward & reverse navigation through the list
 * - Gives users the ability to read & reply to a message in the list
 * - Configurable colors
 *
 * Author: Eric Oulashin (AKA Nightfox)
 * BBS: Digital Distortion
 * BBS address: digdist.bbsindex.com
 *
 * Date       User              Description
 * 2009-04-08 Eric Oulashin     Created
 * 2009-04-09 Eric Oulashin     Updated screen pausing control
 * 2009-04-10 Eric Oulashin     Moved the code into a class in DigitalDistortionDefs.js
 * 2009-04-25 Eric Oulashin     Updated to use new features of the DigDistMsgLister class.
 * 2009-05-03 Eric Oulashin     Moved the DigDistMsgLister class here from
 *                              DigitalDistortionDefs.js.  Also, added a Synchronet
 *                              version check.
 * 2009-05-07 Eric Oulashin     Placed the code for reading a message into its
 *                              own function and updated to allow the user to reply to
 *                              a message.  Updated the traditional interface to
 *                              allow the user to go backwards a pageful of messages.
 * 2009-05-08 Eric Oulashin     Continued to work out small glitches with the prompt-at-end
 *                              interface.  Updated to take an argument (argv[0]) that
 *                              can control whether or not the script code in this
 *                              file is executed.  If someone wants to use the
 *                              DigDistMsgLister class in their own script,
 *                              they could use load("DigitalDistortionMessageLister.js", false)
 *                              and customize the DigDistMsgLister object to their
 *                              own liking.  Added a function that lists a screenful
 *                              of message information,
 *                              DigDistMsgLister_ListScreenfulOfMessages(), and used it
 *                              to simplify the code a bit.
 * 2009-05-09 Eric Oulashin     Updated the prompt-at-end interface to allow navigation
 *                              directly to first & last pages.  Added some functions to
 *                              display user help.  Got the lightbar interface completed.
 * 2009-05-10 Eric Oulashin     Changed all PromptAtPageEnd to Traditional, as I think
 *                              that "Traditional" sounds better.
 * 2009-05-11 Eric Oulashin     Fixed a typo in a variable name for the lightbar interface
 *                              which caused the script to exit when the user pressed F to
 *                              go back to the first page.
 * 2009-05-13 Eric Oulashin     Version 1.02
 *                              Updated to be free of DigitalDistortionDefs.js.  Also,
 *                              added VERSION and VER_DATE.
 * 2009-05-16 Eric Oulashin     Version 1.03
 *                              Improved input speed for the lightbar user interface.
 *                              Added support for the ESC key to quit the listing.
 *                              Bug fix: For the lightbar user interface, when the user
 *                              starts typing a message number, if the user cancels
 *                              the message number input, it would continue with an
 *                              invalid message number.  Now it will just go back to
 *                              the message listing.
 *                              Updated PromptContinueOrReadMsg() to return an
 *                              object with the required properties instead of an
 *                              array, because I think it reads better.
 * 2009-05-18 Eric Oulashin     Version 1.04
 *                              Fixed a bug in DigDistMsgLister_ReadMessage() where
 *                              it was posting the reply message incorrectly, causing
 *                              an error.  Fixed a bug in
 *                              DigDistMsgLister_ListMessages_Lightbar(): If the END
 *                              key was pressed on a page that was only partially
 *                              full of messages, it would go to the last line on
 *                              the screen rather than the last message.
 * 2009-05-23 Eric Oulashin     Version 1.05
 *                              Fixed a bug in the lightbar user interface: If the
 *                              last page of messages fit perfectly on the screen,
 *                              then pressing L to go to the last page would go
 *                              past the last message.
 */

/* Command-line arguments:
   1 (argv[0]): Boolean - Whether or not to run the message list (if false,
                then this file will just provide the DigDistMsgLister class).
*/

load("sbbsdefs.js");

// This script requires Synchronet version 3.13 or higher.
// Exit if the Synchronet version is below the minimum.
if (system.version_num < 31300)
{
	var message = "\1n\1h\1y\1i* Warning:\1n\1h\1w Digital Distortion List Today's Callers "
	             + "requires version \1g3.13\1w or\r\n"
	             + "higher of Synchronet.  This BBS is using version \1g" + system.version
	             + "\1w.  Please notify the sysop.";
	console.crlf();
	console.print(message);
	console.crlf();
	console.pause();
	exit();
}

const VERSION = "1.05";
const VER_DATE = "2009-05-23";

// Keyboard key codes
const CTRL_M = "\x0d";
const KEY_ENTER = CTRL_M;
const ESC_KEY = "\x1b";

// Navigational key characters (for display)
const UP_ARROW = "";
const DOWN_ARROW = "";

// Determine whether or not to execute the message listing code, based
// on the first program argument (a boolean).
var executeThisScript = true;
if (typeof(argv[0]) != "undefined")
	executeThisScript = argv[0];


if (executeThisScript)
{
	// Write a note in the log that the user is listing messages for the sub-board
	bbs.log_str(user.alias + " is listing messages for sub-board: " + bbs.cursub_code);
	//global.log(LOG_INFO, user.alias + " is listing messages for sub-board: " + bbs.cursub_code);
	
	// Create an instance of my DigDistMsgLister class and use it to list the
	// messages.
	var msgLister = new DigDistMsgLister();
	// Note: To change the message listing interface style (traditional prompt at
	// page end or lightbar interface), change the following line:
	//msgLister.interfaceStyle = "Traditional";
	msgLister.interfaceStyle = "Lightbar";
	//msgLister.displayBoardInfoInHeader = true;
	msgLister.displayBoardInfoInHeader = false;
	msgLister.ListMessages(bbs.cursub_code);
}

// End of script execution

///////////////////////////////////////////////////////////////////////////////////
// DigDistMsgLister class stuff

// DigDistMsgLister class constructor: Constructs a
// DigDistMsgLister object, to be used for listing messages
// in a message area.
function DigDistMsgLister()
{
	// this.colors will be an array of colors to use in the message list
	this.colors = getMsgListColors();
	this.msgbase = null; // Will be a MsgBase object

	// String lengths for the columns to write
	// Fixed field widths: Message number, date, and time
	this.MSGNUM_LEN = 4;
	this.DATE_LEN = 10; // i.e., YYYY-MM-DD
	this.TIME_LEN = 8;  // i.e., HH:MM:SS
	// Variable field widths: From, to, and subject (based on a screen width of
	// 80 columns)
	this.FROM_LEN = (console.screen_columns * (15/80)).toFixed(0);
	this.TO_LEN = (console.screen_columns * (15/80)).toFixed(0);
	this.SUBJ_LEN = (console.screen_columns * (23/80)).toFixed(0);
	// If the user's terminal doesn't support ANSI, then use a width of 22 for
	// the subject, to avoid line wrapping.
	if (!console.term_supports(USER_ANSI))
		this.SUBJ_LEN = (console.screen_columns * (22/80)).toFixed(0);

	// Whether or not the user chose to read a message
	this.readAMessage = false;
	// Whether or not the user denied confirmation to read a message
	this.deniedReadingMessage = false;

	// interfaceStyle contains the message listing style and can be one of the
	// following two options:
	// "Traditional": 
	// "Lightbar": 
	this.interfaceStyle = "Traditional";
	
	// displayBoardInfoInHeader specifies whether or not to display
	// the message group and sub-board lines in the header at the
	// top of the screen (an additional 2 lines).
	this.displayBoardInfoInHeader = false;

	// Construct the header format string
	this.sHdrFormatStr = "%" + this.MSGNUM_LEN + "s %-" + this.FROM_LEN + "s %-"
	                   + this.TO_LEN + "s %-" + this.SUBJ_LEN + "s %-"
	                   + this.DATE_LEN + "s %-" + this.TIME_LEN + "s";
	// If the user's terminal doesn't support ANSI, then append a newline to
	// the end of the format string (we won't be able to move the cursor).
	if (!console.term_supports(USER_ANSI))
		this.sHdrFormatStr += "\r\n";

	// Construct the message information format string
	this.sMsgInfoFormatStr = this.colors["msgNum"] + "%" + this.MSGNUM_LEN + "d "
	                       + this.colors["from"]
	                       + "%-" + this.FROM_LEN + "s " + this.colors["to"] + "%-"
	                       + this.TO_LEN + "s " + this.colors["subject"] + "%-"
	                       + this.SUBJ_LEN + "s " + this.colors["date"]
	                       + "%-" + this.DATE_LEN + "s " + this.colors["time"]
	                       + "%-" + this.TIME_LEN + "s";
	// Highlighted message information line (used for the lightbar interface)
	this.sMsgInfoFormatHighlightStr = "4\1h\1y" + "%" + this.MSGNUM_LEN + "d "
	                       + "\1c"
	                       + "%-" + this.FROM_LEN + "s " + "\1c" + "%-"
	                       + this.TO_LEN + "s " + "\1c" + "%-"
	                       + this.SUBJ_LEN + "s " + "\1w"
	                       + "%-" + this.DATE_LEN + "s " + "\1w"
	                       + "%-" + this.TIME_LEN + "s";
	// If the user's terminal doesn't support ANSI, then append a newline to
	// the end of the format string (we won't be able to move the cursor).
	if (!console.term_supports(USER_ANSI))
	{
		this.sMsgInfoFormatStr += "\r\n";
		this.sMsgInfoFormatHighlightStr += "\r\n";
	}

	// Construct the continue prompt text (used for the prompt-at-end interface)
	this.sStartContinuePrompt = "\1n\1c(" + this.colors["contPromptHotkeys"] + "N\1n\1c)"
                          + this.colors["contPromptMain"]
	                      + "ext, \1n\1c(" + this.colors["contPromptHotkeys"] + "L\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "ast, \1n\1c(" + this.colors["contPromptHotkeys"] + "Q\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "uit, message " + this.colors["contPromptHotkeys"] + "#" +
	                      this.colors["contPromptMain"] + ", or " + this.colors["contPromptHotkeys"]
	                      + "?" + this.colors["contPromptMain"] + ": "
	                      + this.colors["contPromptUserInput"];
	this.sContinuePrompt = "\1n\1c(" + this.colors["contPromptHotkeys"] + "N\1n\1c)"
                          + this.colors["contPromptMain"]
	                      + "ext, \1c(" + this.colors["contPromptHotkeys"] + "P\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "rev, \1n\1c(" + this.colors["contPromptHotkeys"] + "F\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "irst, \1n\1c(" + this.colors["contPromptHotkeys"] + "L\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "ast, \1n\1c(" + this.colors["contPromptHotkeys"] + "Q\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "uit, message " + this.colors["contPromptHotkeys"] + "#"
	                      + this.colors["contPromptMain"] + ", or " + this.colors["contPromptHotkeys"]
	                      + "?" + this.colors["contPromptMain"] + ": "
	                      + this.colors["contPromptUserInput"];
	this.sEndContinuePrompt = "\1n\1c(" + this.colors["contPromptHotkeys"] + "P\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "rev, \1n\1c(" + this.colors["contPromptHotkeys"] + "F\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "irst, \1n\1c(" + this.colors["contPromptHotkeys"] + "Q\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "uit, message " + this.colors["contPromptHotkeys"] + "#"
	                      + this.colors["contPromptMain"] + ", or " + this.colors["contPromptHotkeys"]
	                      + "?" + this.colors["contPromptMain"] + ": "
	                      + this.colors["contPromptUserInput"];
	this.sOnlyOnePageContinuePrompt = "\1n\1c(" + this.colors["contPromptHotkeys"] + "Q\1n\1c)"
	                      + this.colors["contPromptMain"]
	                      + "uit, message " + this.colors["contPromptHotkeys"] + "#"
	                      + this.colors["contPromptMain"] + ", or " + this.colors["contPromptHotkeys"]
	                      + "?" + this.colors["contPromptMain"] + ": "
	                      + this.colors["contPromptUserInput"];

	// Construct the line of help text to appear at the bottom of the screen
	// in the lightbar interface
	var helpLineHotkeyColor = "\1r";
	var helpLineNormalColor = "\1b";
	var helpLineParenColor = "\1m";
	this.sLightbarModeHelpLine = "\1n7" + helpLineHotkeyColor + UP_ARROW
	                  + helpLineNormalColor + ", " + helpLineHotkeyColor + DOWN_ARROW
					  + helpLineNormalColor + ", " + helpLineHotkeyColor + "ENTER"
					  + helpLineNormalColor + ", " + helpLineHotkeyColor + "HOME"
					  + helpLineNormalColor + ", " + helpLineHotkeyColor + "END"
					  + helpLineNormalColor + ", " + helpLineParenColor + "("
					  + helpLineHotkeyColor + "N" + helpLineParenColor + ")"
					  + helpLineNormalColor + "ext, " + helpLineParenColor + "("
					  + helpLineHotkeyColor + "P" + helpLineParenColor + ")"
					  + helpLineNormalColor + "rev, " + helpLineParenColor
					  + "(" + helpLineHotkeyColor + "F" + helpLineParenColor
					  + ")" + helpLineNormalColor + "irst, " + helpLineParenColor
					  + "(" + helpLineHotkeyColor + "L" + helpLineParenColor
					  + ")" + helpLineNormalColor + "ast, " + helpLineParenColor
					  + "(" + helpLineHotkeyColor + "Q" + helpLineParenColor
					  + ")" + helpLineNormalColor + "uit, " + helpLineHotkeyColor
					  + "#" + helpLineNormalColor + " or " + helpLineHotkeyColor
					  + "?";

	// Set the function pointers for the object
	this.ListMessages = DigDistMsgLister_ListMessages;
	this.ListMessages_Traditional = DigDistMsgLister_ListMessages_Traditional;
	this.ListMessages_Lightbar = DigDistMsgLister_ListMessages_Lightbar;
	this.PromptContinueOrReadMsg = DigDistMsgLister_PromptContinueOrReadMsg;
	this.WriteScreenTopHeader = DigDistMsgLister_WriteScreenTopHeader;
	this.ReadMessage = DigDistMsgLister_ReadMessage;
	this.PrintMessageInfo = DigDistMsgLister_PrintMessageInfo;
	this.ListScreenfulOfMessages = DigDistMsgLister_ListScreenfulOfMessages;
	this.DisplayHelp = DigDistMsgLister_DisplayHelp;
	this.DisplayTraditionalHelp = DigDistMsgLister_DisplayTraditionalHelp;
	this.DisplayLightbarHelp = DigDistMsgLister_DisplayLightbarHelp;
}
// For the DigDistMsgLister class: Performs the message listing, given a
// sub-board code.
//
// Paramters:
//  pSubBoardCode: The Synchronet sub-board code (string)
function DigDistMsgLister_ListMessages(pSubBoardCode)
{
	// Call the appropriate function, depending on the listing style
	// that is set.
	if (this.interfaceStyle == "Traditional")
	{
		this.ListMessages_Traditional(pSubBoardCode);
	}
	// For page navigation mode, the user's terminal must support ANSI.
	else if ((this.interfaceStyle == "Lightbar") && console.term_supports(USER_ANSI))
	{
		this.ListMessages_Lightbar(pSubBoardCode);
	}
	else
	{
		// Unknown (maybe the user set it to something invalid), or
		// the user's console doesn't support ANSI - Default to the
		// promptAtPageEnd style.
		this.ListMessages_Traditional(pSubBoardCode);
	}
}
// For the DigDistMsgLister class: Performs the message listing, given a
// sub-board code.  This version uses a traditional user interface, prompting
//  the user at the end of each page to continue, quit, or read a message.
//
// Paramters:
//  pSubBoardCode: The Synchronet sub-board code (string)
function DigDistMsgLister_ListMessages_Traditional(pSubBoardCode)
{
	// Reset this.readAMessage and deniedReadingmessage to false, in case the
	// message listing has previously ended with them set to true.
	this.readAMessage = false;
	this.deniedReadingMessage = false;

	// Open the current message sub-board.  If it opened, then list
	// the messages.
	this.msgbase = new MsgBase(pSubBoardCode);
	if (this.msgbase.open())
	{
		// If there are no messages in the current sub-board, then let the
		// user know and exit.
		if (this.msgbase.total_msgs == 0)
		{
			console.clear();
			console.center("\1n\1h\1yThere are no messages in this message sub-board.\r\n\1p");
			return;
		}

		// nMaxLines stores the maximum number of lines to write.  It's the number
		// of rows on the user's screen - 3 to make room for the header line
		// at the top, the question line at the bottom, and 1 extra line at
		// the bottom of the screen so that displaying carriage returns
		// doesn't mess up the position of the header lines at the top.
		var nMaxLines = console.screen_rows-3;
		var nListStartLine = 2; // The first line number on the screen for the message list
		// If we will be displaying the message group and sub-board in the
		// header at the top of the screen (an additional 2 lines), then
		// update nMaxLines and nListStartLine to account for this.
		if (this.displayBoardInfoInHeader)
		{
			nMaxLines -= 2;
			nListStartLine += 2;
		}
		
		// If the user's terminal doesn't support ANSI, then re-calculate
		// nMaxLines - we won't be keeping the headers at the top of the
		// screen.
		if (!console.term_supports(USER_ANSI))
			nMaxLines = console.screen_rows - 2;
		
		// Clear the screen and write the header at the top
		console.clear();
		this.WriteScreenTopHeader();

		// Write the message list
		var topMsgNum = 0;
		var continueOn = true;
		var retvalObj = null;
		var curpos = null; // Current character position
		var lastScreen = false;
		while (continueOn)
		{
			// Go to the top and write the current page of message information,
			// then update curpos.
			console.gotoxy(1, nListStartLine);
			lastScreen = this.ListScreenfulOfMessages(topMsgNum, nMaxLines);
			curpos = console.getxy();
			clearToEOS(curpos.y);
			console.gotoxy(curpos);
			// Prompt the user whether or not to continue or to read a message
			// (by message number).
			retvalObj = this.PromptContinueOrReadMsg((topMsgNum == 0), lastScreen);
			continueOn = retvalObj.continueOn;
			if (continueOn)
			{
				// If the user chose to go to the previous page of listings,
				// then subtract the appropriate number of messages from
				// topMsgNum in order to do so.
				if (retvalObj.userInput == "P")
				{
					topMsgNum -= nMaxLines;
					// If we go past the beginning, then we need to reset
					// msgNum so we'll be at the beginning of the list.
					if (topMsgNum < 0)
						topMsgNum = 0;
				}
				// If the user chose to go to the next page, update
				// topMsgNum appropriately.
				else if (retvalObj.userInput == "N")
					topMsgNum += nMaxLines;
				// First page
				else if (retvalObj.userInput == "F")
					topMsgNum = 0;
				// Last page
				else if (retvalObj.userInput == "L")
					topMsgNum = this.msgbase.total_msgs - (this.msgbase.total_msgs % nMaxLines);
				// Display help
				else if (retvalObj.userInput == "?")
				{
					console.clear();
					this.DisplayHelp(true);
					console.clear();
					this.WriteScreenTopHeader();
				}
				
				// If the user chose to read a message or denied confirmation, then:
				// - Re-draw the column headers at the top of the screen.
				// - Subtract nMaxLines from msgNum so that this script displays
				//   the same page where the user left off.
				if (this.readAMessage || this.deniedReadingMessage)
				{
					if (console.term_supports(USER_ANSI))
						this.WriteScreenTopHeader();
				}
				this.readAMessage = false;
				this.deniedReadingMessage = false;

				// If the user's terminal doesn't support ANSI, then adjust
				// nMaxLines to 1 less than the number of screen rows, because
				// after the first page, we no longer need to display the message
				// list header line.
				if (!console.term_supports(USER_ANSI))
					nMaxLines = console.screen_rows - 1;
			}
		}
	
		this.msgbase.close();
	}
	// Make sure color highlighting is turned off
	console.print("\1n");
}
// For the DigDistMsgLister class: Performs the message listing, given a
// sub-board code.  This verison uses a lightbar interface for message
// navigation.
//
// Paramters:
//  pSubBoardCode: The Synchronet sub-board code (string)
function DigDistMsgLister_ListMessages_Lightbar(pSubBoardCode)
{
	// This method is only supported if the user's terminal supports
	// ANSI.
	if (!console.term_supports(USER_ANSI))
	{
		console.print("\r\n\1h\1yError: An ANSI terminal is required for this operation.\1n\1w\r\n");
		console.print("Press any key:");
		console.getkey();
		return;
	}

	// Reset this.readAMessage and deniedReadingMessage to false, in case the
	// message listing has previously ended with them set to true.
	this.readAMessage = false;
	this.deniedReadingMessage = false;

	// Open the current message sub-board.  If it opened, then list
	// the messages.
	this.msgbase = new MsgBase(pSubBoardCode);
	if (this.msgbase.open())
	{
		// If there are no messages in the current sub-board, then let the
		// user know and exit.
		if (this.msgbase.total_msgs == 0)
		{
			console.clear();
			console.center("\1n\1h\1yThere are no messages in this message sub-board.\r\n\1p");
			return;
		}

		// This function will be used for displaying the help line at
		// the bottom of the screen.
		function DisplayHelpLine(pHelpLineText)
		{
			console.gotoxy(1, console.screen_rows);
			console.print(pHelpLineText);
			console.cleartoeol();
		}

		// nMaxLines stores the maximum number of lines to write.  It's the number
		// of rows on the user's screen - 3 to make room for the header line
		// at the top, the question line at the bottom, and 1 extra line at
		// the bottom of the screen so that displaying carriage returns
		// doesn't mess up the position of the header lines at the top.
		var nMaxLines = console.screen_rows-2;
		var nListStartLine = 2; // The first line number on the screen for the message list
		// If we will be displaying the message group and sub-board in the
		// header at the top of the screen (an additional 2 lines), then
		// update nMaxLines and nListStartLine to account for this.
		if (this.displayBoardInfoInHeader)
		{
			nMaxLines -= 2;
			nListStartLine += 2;
		}
		
		// Clear the screen and write the header at the top
		console.clear();
		this.WriteScreenTopHeader();
		DisplayHelpLine(this.sLightbarModeHelpLine);

		// Set the cursor position to the first line where the list should start
		console.gotoxy(1, nListStartLine);
		
		// Write the first page of message listings.
		var topMessageNum = 0;
		var lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);

		// Move the cursor to the first message line
		console.gotoxy(1, nListStartLine);
		var curpos = console.getxy(); // Keeps track of the current cursor position
		// User input loop
		var selectedMsgNum = 1;
		var bottomMsgNum = 0;
		var userInput = "";
		var msgHeader = null;
		var continueOn = true;
		while (continueOn)
		{
			// Calculate the message number (0-based) of the message
			// appearing on the bottom of the screen.
			bottomMsgNum = topMessageNum + nMaxLines;
			if (bottomMsgNum > this.msgbase.total_msgs)
				bottomMsgNum = this.msgbase.total_msgs;

			// Write the current message information with highlighting colors
			msgHeader = this.msgbase.get_msg_header(true, selectedMsgNum-1, true);
			this.PrintMessageInfo(msgHeader, true);
			console.gotoxy(curpos); // Make sure the cursor is still in the right place

			// Take an appropriate action based on the user's keypress.
			userInput = console.getkey().toString().toUpperCase();
			if ((userInput == "Q") || (userInput == ESC_KEY))
			{
				// Quit
				continueOn = false;
				break;
			}
			else if (userInput == "?")
			{
				// Display help
				console.print("\1n"); // Remove any background color
				console.clear();
				this.DisplayHelp(true);

				// Re-draw the message list on the screen
				console.clear();
				this.WriteScreenTopHeader();
				DisplayHelpLine(this.sLightbarModeHelpLine);
				console.gotoxy(1, nListStartLine);
				lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
				console.gotoxy(curpos); // Put the cursor back where it should be
			}
			else if ((userInput == KEY_UP) && (selectedMsgNum > 1))
			{
				// Print the current message information with regular colors
				this.PrintMessageInfo(msgHeader, false);

				--selectedMsgNum;

				// If the current screen row is above the first line allowed, then
				// move the cursor up one row.
				if (curpos.y > nListStartLine)
				{
					console.gotoxy(1, curpos.y-1);
					curpos.x = 1;
					--curpos.y;
				}
				else
				{
					// Go onto the previous page, with the cursor highlighting
					// the last message on the page.
					console.gotoxy(1, nListStartLine);
					topMessageNum = selectedMsgNum-nMaxLines;
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					console.gotoxy(1, nListStartLine+nMaxLines-1);
					curpos.x = 1;
					curpos.y = nListStartLine+nMaxLines-1;
				}
			}
			else if ((userInput == KEY_DOWN) && (selectedMsgNum < this.msgbase.total_msgs))
			{
				// Print the current message information with regular colors
				this.PrintMessageInfo(msgHeader, false);

				++selectedMsgNum;

				// If the current screen row is below the last line allowed, then
				// move the cursor down one row.
				if (curpos.y < nListStartLine+nMaxLines-1)
				{
					console.gotoxy(1, curpos.y+1);
					curpos.x = 1;
					++curpos.y;
				}
				else
				{
					// Go onto the next page, with the cursor highlighting
					// the first message on the page.
					console.gotoxy(1, nListStartLine);
					topMessageNum = selectedMsgNum-1;
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					// If we were on the last page, then clear the screen from
					// the current line to the end of the screen.
					if (lastPage)
					{
						curpos = console.getxy();
						clearToEOS(curpos.y);
						// Make sure the help line is still there
						DisplayHelpLine(this.sLightbarModeHelpLine);
					}

					// Move the cursor to the top of the list
					console.gotoxy(1, nListStartLine);
					curpos.x = 1;
					curpos.y = nListStartLine;
				}
			}
			else if (userInput == KEY_HOME)
			{
				// Print the current message information with regular colors
				this.PrintMessageInfo(msgHeader, false);
				// Go to the first message of the current page
				selectedMsgNum -= (curpos.y - nListStartLine);
				// Move the cursor to the first message line
				console.gotoxy(1, nListStartLine);
				curpos.x = 1;
				curpos.y = nListStartLine;
			}
			else if (userInput == KEY_END)
			{
				// Print the current message information with regular colors
				this.PrintMessageInfo(msgHeader, false);
				// Update the selected message #
				selectedMsgNum = bottomMsgNum;
				// Go to the last message of the current page
				curpos.y = nListStartLine + bottomMsgNum - topMessageNum - 1;
				console.gotoxy(curpos);
			}
			else if (userInput == KEY_ENTER)
			{
				var originalCurpos = console.getxy();

				// Allow the user to read the current message.
				// Confirm with the user whether to read the message.
				var sReadMsgConfirmText = this.colors["readMsgConfirm"]
										 + "Read message "
										 + this.colors["readMsgConfirmNumber"]
										 + selectedMsgNum + this.colors["readMsgConfirm"]
										 + ": Are you sure";
				console.gotoxy(1, console.screen_rows);
				console.print("\1n");
				console.clearline();
				if (console.yesno(sReadMsgConfirmText))
				{
					this.readAMessage = true;
					console.clear();
					this.ReadMessage(selectedMsgNum-1);
				}
				else
					this.deniedReadingMessage = true;

				// Ask the user if  they want to continue reading messages
				continueOn = console.yesno(this.colors["afterReadMsg_ListMorePrompt"] +
										   "Continue listing messages");
				// If the user chose to continue reading messages, then refresh
				// the screen.  Even if the user chooses not to read the message,
				// the screen needs to be re-drawn so it appears properly.
				if (continueOn)
				{
					console.clear();
					this.WriteScreenTopHeader();
					DisplayHelpLine(this.sLightbarModeHelpLine);
					console.gotoxy(1, nListStartLine);
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					console.gotoxy(originalCurpos); // Put the cursor back where it should be
				}
			}
			// TODO: Figure out how to get PageDown and PageUp from the user.
			// For now, this script uses N and P.
			else if (userInput == "N")
			{
				// Next page
				if (!lastPage)
				{
					topMessageNum += nMaxLines;
					selectedMsgNum = topMessageNum+1;
					console.gotoxy(1, nListStartLine);
					curpos.x = 1;
					curpos.y = nListStartLine;
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					
					// If we were on the last page, then clear the screen from
					// the current line to the end of the screen.
					if (lastPage)
					{
						curpos = console.getxy();
						clearToEOS(curpos.y);
						// Make sure the help line is still there
						DisplayHelpLine(this.sLightbarModeHelpLine);
					}

					// Move the cursor back to the first message info line
					console.gotoxy(1, nListStartLine);
					curpos.x = 1;
					curpos.y = nListStartLine;
				}
			}
			else if (userInput == "P")
			{
				// Previous page
				if (topMessageNum > 0)
				{
					topMessageNum -= (nMaxLines);
					selectedMsgNum = topMessageNum+1;
					console.gotoxy(1, nListStartLine);
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					console.gotoxy(1, nListStartLine);
					curpos.x = 1;
					curpos.y = nListStartLine;
				}
			}
			// First page
			else if (userInput == "F")
			{
				if (topMessageNum > 0)
				{
					topMessageNum = 0;
					selectedMsgNum = topMessageNum+1;
					console.gotoxy(1, nListStartLine);
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					console.gotoxy(1, nListStartLine);
					curpos.x = 1;
					curpos.y = nListStartLine;
				}
			}
			// Last page
			else if (userInput == "L")
			{
				if (!lastPage)
				{
					topMessageNum = this.msgbase.total_msgs - (this.msgbase.total_msgs % nMaxLines);
					// If topMessageNum is beyond the last message in the sub-board,
					// then move back a full page of messages.
					if (topMessageNum >= this.msgbase.total_msgs)
                  topMessageNum = this.msgbase.total_msgs - nMaxLines;
					selectedMsgNum = topMessageNum+1;
					console.gotoxy(1, nListStartLine);
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					// If we were on the last page, then clear the screen from
					// the current line to the end of the screen.
					if (lastPage)
					{
						curpos = console.getxy();
						clearToEOS(curpos.y);
						// Make sure the help line is still there
						DisplayHelpLine(this.sLightbarModeHelpLine);
					}

					// Move the cursor back to the first message info line
					console.gotoxy(1, nListStartLine);
					curpos.x = 1;
					curpos.y = nListStartLine;
				}
			}
			// Numeric digit: The start of a message number
			else if (userInput.match(/[0-9]/))
			{
				var originalCurpos = console.getxy();

				// Put the user's input back in the input buffer to
				// be used for getting the rest of the message number.
				console.ungetstr(userInput);
				// Move the cursor to the bottom of the screen and
				// prompt the user for the message number.
				console.gotoxy(1, console.screen_rows);
				console.print("\1n");
				console.clearline();
				console.print("\1cMessage #: \1h");
				userInput = console.getnum(this.msgbase.total_msgs);
				if (userInput > 0)
				{
					// Confirm with the user whether to read the message
					var sReadMsgConfirmText = this.colors["readMsgConfirm"]
											 + "Read message "
											 + this.colors["readMsgConfirmNumber"]
											 + userInput + this.colors["readMsgConfirm"]
											 + ": Are you sure";
					if (console.yesno(sReadMsgConfirmText))
					{
						this.readAMessage = true;
						this.ReadMessage(userInput-1);
					}
					else
						this.deniedReadingMessage = true;
	
					// Prompt the user whether or not to continue listing
					// messages.
					continueOn = console.yesno(this.colors["afterReadMsg_ListMorePrompt"] +
											   "Continue listing messages");
				}

				// If the user chose to continue listing messages, then re-draw
				// the screen.
				if (continueOn)
				{
					console.clear();
					this.WriteScreenTopHeader();
					DisplayHelpLine(this.sLightbarModeHelpLine);
					console.gotoxy(1, nListStartLine);
					lastPage = this.ListScreenfulOfMessages(topMessageNum, nMaxLines);
					console.gotoxy(originalCurpos); // Put the cursor back where it should be
				}
			}
		}

		this.msgbase.close();
	}
	// Make sure color highlighting is turned off
	console.print("\1n");
}
// For the DigDistMsgListerClass: Prints a line of information about
// a message.
//
// Parameters:
//  pMsgHeader: The message header object, returned by MsgBase.get_msg_header().
//  pHighlight (optional): Boolean, whether or not to highlight the line or
//                         use the standard colors.
function DigDistMsgLister_PrintMessageInfo(pMsgHeader, pHighlight)
{
	// pMsgHeader must be a valid object.
	if (typeof(pMsgHeader) == "undefined")
		return;
	if (pMsgHeader == null)
		return;

	var highlight = false;
	if (typeof(pHighlight) != "undefined")
		highlight = pHighlight;
	

	// Parse the message date into the day name, day,
	// month name, year, hour, minute, and second.
	var myDateArr = parseSynchronetMsgDate(pMsgHeader["date"]);

	// Construct the date in YYYY-MM-DD format
	var sDate = myDateArr["year"] + "-" + myDateArr["month"] + "-"
               + myDateArr["day"];

	// Construct the time in HH:MM:SS format
	var sTime = myDateArr["hour"] + ":" + myDateArr["min"] + ":"
              + myDateArr["sec"];

	// Write the message header information.
	// Note: The message header has the following fields:
	// 'number': The message number
	// 'offset': The message offset
	// 'to': Who the message is directed to (string)
	// 'from' Who wrote the message (string)
	// 'subject': The message subject (string)
	// 'date': The date - Full text (string)
	// To access one of these, use brackets; i.e., msgHeader['to']
	if (highlight)
	{
		printf(this.sMsgInfoFormatHighlightStr,
			   //pMsgHeader["number"],
			   pMsgHeader["offset"]+1,
			   pMsgHeader['from'].substr(0, this.FROM_LEN),
			   pMsgHeader['to'].substr(0, this.TO_LEN),
			   pMsgHeader['subject'].substr(0, this.SUBJ_LEN),
			   sDate, sTime);
	}
	else
	{
		printf(this.sMsgInfoFormatStr,
			   //pMsgHeader["number"],
			   pMsgHeader["offset"]+1,
			   pMsgHeader['from'].substr(0, this.FROM_LEN),
			   pMsgHeader['to'].substr(0, this.TO_LEN),
			   pMsgHeader['subject'].substr(0, this.SUBJ_LEN),
			   sDate, sTime);
	}
}
// For the DigDistMsgListerClass: Prompts the user to continue or read
// a message (by number).
//
// Parameters:
//  pStart: Whether or not we're on the first page (true or false)
//  pEnd: Whether or not we're at the last page (true or false)
//
// Return value: An array containing the following elements:
//               0: Boolean, whether or not the user wants to continue
//                  listing the messages (true or false).
//               1: The user's input
function DigDistMsgLister_PromptContinueOrReadMsg(pStart, pEnd)
{
	var continueOn = true;

	// Prompt the user whether or not to continue or to read a message
	// (by message number).  If we're at the end of the list, then the
	// prompt text will need to be different.
	var userInput = "";
	var allowedKeys = "?"; // ? = help
	if (pStart && pEnd)
	{
		// This is the only page.
		console.print(this.sOnlyOnePageContinuePrompt);
		// Get input from the user.  Allow only Q (quit).
		allowedKeys += "Q";
	}
	else if (pStart)
	{
		// We're on the first page.
		console.print(this.sStartContinuePrompt);
		// Get input from the user.  Allow only L (last), N (next), or Q (quit).
		allowedKeys += "LNQ";
	}
	else if (pEnd)
	{
		// We're on the last page.
		console.print(this.sEndContinuePrompt);
		// Get input from the user.  Allow only F (first), P (previous), or Q (quit).
		allowedKeys += "FPQ";
	}
	else
	{
		// We're neither on the first nor last page.  Allow F (first), L (last),
		// N (next), P (previous), or Q (quit).
		console.print(this.sContinuePrompt);
		allowedKeys += "FLNPQ";
	}
	// Get the user's input.  Allow the keys in allowedKeys or a number from 1
	// to the highest message number.
	userInput = console.getkeys(allowedKeys, this.msgbase.total_msgs).toString();
	if (userInput == "Q")
		continueOn = false;

	// If the user has typed all numbers, then read that message.
	if ((userInput != "") && /^[0-9]+$/.test(userInput))
	{
		// Confirm with the user whether to read the message
		var sReadMsgConfirmText = this.colors["readMsgConfirm"]
								 + "Read message "
								 + this.colors["readMsgConfirmNumber"]
								 + userInput + this.colors["readMsgConfirm"]
								 + ": Are you sure";
		if (console.yesno(sReadMsgConfirmText))
		{
			this.readAMessage = true;
			this.ReadMessage(userInput-1);
		}
		else
			this.deniedReadingMessage = true;

		// Prompt the user whether or not to continue listing
		// messages.
		continueOn = console.yesno(this.colors["afterReadMsg_ListMorePrompt"] +
								   "Continue listing messages");
	}

	// Make sure color highlighting is turned off
	console.print("\1n");

	// Fill a return object with the required values, and return it.
	var returnObj = new Object();
	returnObj.continueOn = continueOn;
	returnObj.userInput = userInput;
	return returnObj;
}
// For the DigDistMsgLister Class: Given a message number of a message in the
// current message area, shows the message to the user and allows the user to
// respond.
//
// Parameters:
//  pMessageNumber: The number of the message in the current message area
function DigDistMsgLister_ReadMessage(pMessageNumber)
{
	// Get the message header
	var msgHeader = this.msgbase.get_msg_header(true, pMessageNumber);
	
	// Parse the message date into the day name, day,
	// month name, year, hour, minute, and second.
	var myDateArr = parseSynchronetMsgDate(msgHeader["date"]);

	// Construct the date in YYYY-MM-DD format
	var sDate = myDateArr["year"] + "-" + myDateArr["month"] + "-"
               + myDateArr["day"];

	// Construct the time in HH:MM:SS format
	var sTime = myDateArr["hour"] + ":" + myDateArr["min"] + ":"
              + myDateArr["sec"];

	// Show the message.
	// Note: The message header has the following fields:
	// 'number': The message number
	// 'offset': The message offset
	// 'to': Who the message is directed to (string)
	// 'from' Who wrote the message (string)
	// 'subject': The message subject (string)
	// 'date': The date - Full text (string)
	console.print("\1n\1gFrom: \1h" + msgHeader["from"]);
	console.crlf();
	console.print("\1n\1gTo: \1h" + msgHeader["to"]);
	console.crlf();
	console.print("\1n\1gSubj: \1h" + msgHeader["subject"]);
	console.crlf();
	console.print("\1n\1gDate: \1h" + sDate + " " + sTime);
	console.crlf();
	console.print("\1n\1gMessage:");
	console.crlf();
	console.print("\1h========");
	console.crlf();
	console.print(this.colors["msgBody"]);
	console.print(this.msgbase.get_msg_body(true, pMessageNumber));

	// Allow the user to reply to the message.
	// Note: console.noyes() returns false when the user selects Yes.
	if (!console.noyes("\1n\1cEnd of message.  Reply"))
	{
		// No special behavior in the reply
		var replyMode = WM_NONE;

		// Write QUOTES.TXT in the node directory to allow the user to quote
		// the original message.
		var msgBody = this.msgbase.get_msg_body(true, pMessageNumber);
		msgBody = quote_msg(msgBody, console.screen_columns-1);
		var quoteFile = new File(system.node_dir + "QUOTES.TXT");
		if (quoteFile.open("w"))
		{
			quoteFile.write(msgBody);
			quoteFile.close();
			// Let the user quote in the reply
			replyMode = WM_QUOTE;
		}

		// If posting in a local group, then the 'from' and 'to' names
		// in msgHeader must be swapped in order to have the correct 'to'
		// name in the reply.
		if (msgHeader.from_net_type == NET_NONE)
		{
			var fromBackup = msgHeader.from;
			msgHeader.from = msgHeader.to;
			msgHeader.to = fromBackup;
		}
		// Let the user post a reply.
		bbs.post_msg(null, replyMode, msgHeader);

		// Remove the quote file from disk
		quoteFile.remove();
	}
}
// For the DigDistMsgLister Class: Writes the message columns at the top of the
// screen.
function DigDistMsgLister_WriteScreenTopHeader()
{
	console.home();

	// If we will be displaying the message group and sub-board in the
	// header at the top of the screen (an additional 2 lines), then
	// update nMaxLines and nListStartLine to account for this.
	if (this.displayBoardInfoInHeader && console.term_supports(USER_ANSI))
	{
		var curpos = console.getxy();
		// Display the current message group
		printf(this.colors["headerMsgGroupText"] + "Current msg group: " + this.colors["headerMsgGroupName"] + "%s", msg_area.grp_list[bbs.curgrp].description);
		console.cleartoeol(); // Fill to the end of the line with the current colors
		++curpos.y;
		console.gotoxy(curpos);
		// Display the current message sub-board
		printf(this.colors["headerSubBoardText"] + "Current sub-board: " + this.colors["headerMsgSubBoardName"] + "%s", msg_area.grp_list[bbs.curgrp].sub_list[bbs.cursub].name);
		console.cleartoeol(); // Fill to the end of the line with the current colors
		++curpos.y;
		console.gotoxy(curpos);
	}

	// Write the message listing column headers
	printf(this.colors["listColHeader"] + this.sHdrFormatStr, "Msg#", "From", "To", "Subject", "Date", "Time");

	// Set the normal text attribute
	console.print("\1n");
}
// For the DigDistMsgLister Class: Lists a screenful of message header information.
//
// Parameters:
//  pTopMsgNum: The message number of the top message (0-based)
//  pMaxLines: The maximum number of lines to output to the screen
//
// Return value: Boolean, whether or not the last message output to the
//               screen is the last message in the sub-board.
function DigDistMsgLister_ListScreenfulOfMessages(pTopMsgNum, pMaxLines)
{
	var msgHeader = null;
	var endMsgNum = pTopMsgNum + pMaxLines;
	var curpos = console.getxy();
	var msgNum = 0;
	for (msgNum = pTopMsgNum; (msgNum < this.msgbase.total_msgs) && (msgNum < endMsgNum); ++msgNum)
	{
		// The following line which sets console.line_counter to 0 is a
		// kludge to disable Synchronet's automatic pausing after a
		// screenful of text, so that this script can have more control
		// over screen pausing.
		console.line_counter = 0;

		// Get the message header (it will be a MsgHeader object) and
		// display it.
		msgHeader = this.msgbase.get_msg_header(true, msgNum, true);
		if (msgHeader == null)
			continue;

		// Display the message info
		this.PrintMessageInfo(msgHeader, false);
		++curpos.y;
		console.gotoxy(curpos);
	}
	
	// Return whether or not the last message written to the screen was
	// the last message in the sub-board.
	return (msgNum == this.msgbase.total_msgs);
}
// For the DigDistMsgLister Class: Displays general help.
//
// Parameters:
//  pPauseAtEnd: Boolean, whether or not to pause at the end.
function DigDistMsgLister_DisplayHelp(pPauseAtEnd)
{
	console.print("\1n" + this.colors["helpScreen"]);
	DisplayProgramInfo();

	// Display help specific to which interface is being used.
	if (this.interfaceStyle == "Traditional")
		this.DisplayTraditionalHelp(false);
	else if (this.interfaceStyle == "Lightbar")
		this.DisplayLightbarHelp(false);

	// If pPauseAtEnd is true, then output a newline and
	// prompt the user whether or not to continue.
	if (pPauseAtEnd)
	{
		console.crlf();
		console.pause();
	}
}
// For the DigDistMsgLister Class: Displays help specific to the page-end
// interface.
//
// Parameters:
//  pDisplayHeader: Whether or not to display a help header at the beginning
//  pPauseAtEnd: Boolean, whether or not to pause at the end.
function DigDistMsgLister_DisplayTraditionalHelp(pDisplayHeader, pPauseAtEnd)
{
	console.print("\1n" + this.colors["helpScreen"]);

	// If pDisplayHeader is true, then display the program information.
	if (pDisplayHeader)
		DisplayProgramInfo();

	console.print("Page navigation and message selection");
	console.crlf();
	console.print("The message lister will display a page of message header information.  At");
	console.crlf();
	console.print("the end of each page, a prompt is displayed, allowing you to navigate to");
	console.crlf();
	console.print("the next page, previous page, first page, or the last page.  If you would");
	console.crlf();
	console.print("like to read a message, you may type the message number, followed by");
	console.crlf();
	console.print("the enter key if the message number is short.  To quit the listing, press");
	console.crlf();
	console.print("the Q key.");
	console.crlf();
	console.crlf();
	console.print("The following is a summary of the keyboard commands:");
	console.crlf();
	console.print("\1n\1h\1cN" + this.colors["helpScreen"] + ": Go to the next page");
	console.crlf();
	console.print("\1n\1h\1cP" + this.colors["helpScreen"] + ": Go to the previous page");
	console.crlf();
	console.print("\1n\1h\1cF" + this.colors["helpScreen"] + ": Go to the first page");
	console.crlf();
	console.print("\1n\1h\1cL" + this.colors["helpScreen"] + ": Go to the last page");
	console.crlf();
	console.print("\1n\1h\1cNumber" + this.colors["helpScreen"] + ": Read the message corresponding with that number");
	console.crlf();
	console.print("\1n\1h\1cQ" + this.colors["helpScreen"] + ": Quit");
	console.crlf();
	console.print("\1n\1h\1c?" + this.colors["helpScreen"] + ": Show this help screen");
	
	// If pPauseAtEnd is true, then output a newline and
	// prompt the user whether or not to continue.
	if (pPauseAtEnd)
	{
		console.crlf();
		console.pause();
	}
}
// For the DigDistMsgLister Class: Displays help specific to the lightbar
// interface.
//
// Parameters:
//  pDisplayHeader: Whether or not to display a help header at the beginning
//  pPauseAtEnd: Boolean, whether or not to pause at the end.
function DigDistMsgLister_DisplayLightbarHelp(pDisplayHeader, pPauseAtEnd)
{
	console.print("\1n" + this.colors["helpScreen"]);
	
	// If pDisplayHeader is true, then display the program information.
	if (pDisplayHeader)
		DisplayProgramInfo();

	console.print("Lightbar interface: Page navigation and message selection");
	console.crlf();
	console.print("The message lister will display a page of message header information.  You");
	console.crlf();
	console.print("may use the up and down arrows to navigate the list of messages.  The");
	console.crlf();
	console.print("currently-selected message will be highlighted as you navigate through");
	console.crlf();
	console.print("the list.  To read a message, navigate to the desired message and press");
	console.crlf();
	console.print("the enter key.  You can also read a message by typing its message number.");
	console.crlf();
	console.print("To quit out of the message list, press the Q key.");
	console.crlf();
	console.crlf();
	console.print("The following is a summary of the keyboard commands:");
	console.crlf();
	console.print("\1n\1h\1cDown arrow" + this.colors["helpScreen"] + ": Move the cursor down/select the next message");
	console.crlf();
	console.print("\1n\1h\1cUp arrow" + this.colors["helpScreen"] + ": Move the cursor up/select the previous message");
	console.crlf();
	console.print("\1n\1h\1cN" + this.colors["helpScreen"] + ": Go to the next page");
	console.crlf();
	console.print("\1n\1h\1cP" + this.colors["helpScreen"] + ": Go to the previous page");
	console.crlf();
	console.print("\1n\1h\1cF" + this.colors["helpScreen"] + ": Go to the first page");
	console.crlf();
	console.print("\1n\1h\1cL" + this.colors["helpScreen"] + ": Go to the last page");
	console.crlf();
	console.print("\1n\1h\1cNumber" + this.colors["helpScreen"] + ": Read the message corresponding with that number");
	console.crlf();
	console.print("\1n\1h\1cQ" + this.colors["helpScreen"] + ": Quit");
	console.crlf();
	console.print("\1n\1h\1c?" + this.colors["helpScreen"] + ": Show this help screen");
	
	// If pPauseAtEnd is true, then output a newline and
	// prompt the user whether or not to continue.
	if (pPauseAtEnd)
	{
		console.crlf();
		console.pause();
	}
}

///////////////////////////////////////////////////////////////////////////////////
// Functions

// Displays the program information.
function DisplayProgramInfo()
{
	console.center("Digital Distortion Message Lister");
	console.center("=================================");
	console.center("Version " + VERSION);
	console.center(VER_DATE);
	console.crlf();
	console.center("Written by Eric Oulashin of Digital Distortion BBS.");
	console.crlf();
	console.crlf();
}

// This function returns an array of colors used in the DigDistMessageLister class.
function getMsgListColors()
{
	var colorArray = new Array();
	
	// Header line: "Current msg group:"
	colorArray["headerMsgGroupText"] = "\1n4\1c"; // Normal cyan on blue background
	//colorArray["headerMsgGroupText"] = "\1n4\1w"; // Normal white on blue background

	// Header line: Message group name
	colorArray["headerMsgGroupName"] = "\1h\1c"; // High cyan
	//colorArray["headerMsgGroupName"] = "\1h\1w"; // High white

	// Header line: "Current sub-board:"
	colorArray["headerSubBoardText"] = "\1n4\1c"; // Normal cyan on blue background
	//colorArray["headerSubBoardText"] = "\1n4\1w"; // Normal white on blue background

	// Header line: Message sub-board name
	colorArray["headerMsgSubBoardName"] = "\1h\1c"; // High cyan
	//colorArray["headerMsgSubBoardName"] = "\1h\1w"; // High white
	// Line with column headers
	//colorArray["listColHeader"] = "\1h\1w"; // High white (keep blue background)
	colorArray["listColHeader"] = "\1n\1h\1w"; // High white on black background
	//colorArray["listColHeader"] = "\1h\1c"; // High cyan (keep blue background)
	//colorArray["listColHeader"] = "4\1h\1y"; // High yellow (keep blue background)

	// Header separator line
	colorArray["headerSeparatorLine"] = "\1h\1w";

	// Message information
	colorArray["msgNum"] = "\1n\1h\1y";
	//colorArray["msgNum"] = "\1n\1h\1w";
	colorArray["from"] = "\1n\1c";
	colorArray["to"] = "\1n\1c";
	colorArray["subject"] = "\1n\1c";
	colorArray["date"] = "\1h\1b";
	colorArray["time"] = "\1h\1b";

	// Continue prompt colors
	colorArray["contPromptMain"] = "\1n\1g"; // Main text color
	colorArray["contPromptHotkeys"] = "\1h\1c"; // Hotkey color
	colorArray["contPromptUserInput"] = "\1h\1g"; // User input color

	// Message body color
	colorArray["msgBody"] = "\1n\1w";

	// Read message confirmation colors
	colorArray["readMsgConfirm"] = "\1n\1c";
	colorArray["readMsgConfirmNumber"] = "\1h\1c";

	// Prompt for continuing to list messages after reading a message
	colorArray["afterReadMsg_ListMorePrompt"] = "\1n\1c";
	
	// Help screen text color
	colorArray["helpScreen"] = "\1n\1h\1w";
	
	return colorArray;
}

// This function parses a Syncrhonet message date & time and returns an
// array containing the date & time elements ('year', 'month', 'day',
// 'hour', 'min', 'sec', 'dayName', 'monthName').  All numbers will be
// zero-padded.  Note that the array will be created with 'new'.
function parseSynchronetMsgDate(pSynchronetMsgDate)
{
	var myDateArray = new Array();

	// A day number may be 1 or 2 digits, and we'll need to
	// deal with that.  i.e.:
	//   Fri, 29 Feb 2008 06:01:00
	//   Sat, 1 Mar 2008 09:27:00
	// If the character at position 6 is a space, then we know
	// the date has a single-digit day.  Otherwise, it has a
	// 2-digit day.
	if (pSynchronetMsgDate.charAt(6) == ' ')
	{
		// 1-digit day, i.e.: Sat, 1 Mar 2008 09:27:00
		myDateArray["dayName"] = pSynchronetMsgDate.substr(0, 3);
		myDateArray["day"] = pSynchronetMsgDate.substr(5, 1);
		myDateArray["monthName"] = pSynchronetMsgDate.substr(7, 3);
		myDateArray["year"] = pSynchronetMsgDate.substr(11, 4);
		myDateArray["hour"] = pSynchronetMsgDate.substr(16, 2);
		myDateArray["min"] = pSynchronetMsgDate.substr(19, 2);
		myDateArray["sec"] = pSynchronetMsgDate.substr(22, 2);
	}
	else
	{
		// 2-digit day, i.e.: Fri, 29 Feb 2008 06:01:00
		myDateArray["dayName"] = pSynchronetMsgDate.substr(0, 3);
		myDateArray["day"] = pSynchronetMsgDate.substr(5, 2);
		myDateArray["monthName"] = pSynchronetMsgDate.substr(8, 3);
		myDateArray["year"] = pSynchronetMsgDate.substr(12, 4);
		myDateArray["hour"] = pSynchronetMsgDate.substr(17, 2);
		myDateArray["min"] = pSynchronetMsgDate.substr(20, 2);
		myDateArray["sec"] = pSynchronetMsgDate.substr(23, 2);
	}
	
	myDateArray["month"] = getMonthNum(myDateArray["monthName"]);
	
	// Zero-pad the month number
	if (myDateArray["month"] < 10)
		myDateArray["month"] = "0" + myDateArray["month"];
	// Zero-pad the day number
	if (myDateArray["day"] < 10)
		myDateArray["day"] = "0" + myDateArray["day"];

	return myDateArray;
}

// This function returns the month number (1-based) from a capitalized
// month name.
//
// Parameters:
//  pMonthName: The name of the month
//
// Return value: The number of the month (1-12).
function getMonthNum(pMonthName)
{
	var monthNum = 1;

	if (pMonthName.substr(0, 3) == "Jan")
	{
		monthNum = 1;
	}
	else if (pMonthName.substr(0, 3) == "Feb")
	{
		monthNum = 2;
	}
	else if (pMonthName.substr(0, 3) == "Mar")
	{
		monthNum = 3;
	}
	else if (pMonthName.substr(0, 3) == "Apr")
	{
		monthNum = 4;
	}
	else if (pMonthName.substr(0, 3) == "May")
	{
		monthNum = 5;
	}
	else if (pMonthName.substr(0, 3) == "Jun")
	{
		monthNum = 6;
	}
	else if (pMonthName.substr(0, 3) == "Jul")
	{
		monthNum = 7;
	}
	else if (pMonthName.substr(0, 3) == "Aug")
	{
		monthNum = 8;
	}
	else if (pMonthName.substr(0, 3) == "Sep")
	{
		monthNum = 9;
	}
	else if (pMonthName.substr(0, 3) == "Oct")
	{
		monthNum = 10;
	}
	else if (pMonthName.substr(0, 3) == "Nov")
	{
		monthNum = 11;
	}
	else if (pMonthName.substr(0, 3) == "Dec")
	{
		monthNum = 12;
	}

	return monthNum;
}

// Clears each line from a given line to the end of the screen.
//
// Parameters:
//  pStartLineNum: The line number to start at (1-based)
function clearToEOS(pStartLineNum)
{
	if (typeof(pStartLineNum) == "undefined")
		return;
	if (pStartLineNum == null)
		return;
	
	for (var lineNum = pStartLineNum; lineNum <= console.screen_rows; ++lineNum)
	{
		console.gotoxy(1, lineNum);
		console.clearline();
	}
}